@@ -1,7 +1,7 @@
 /*-
  * SPDX-License-Identifier: BSD-3-Clause
  *
- * Copyright (c) 2018-2021  Joachim Wiberg <troglobit@gmail.com>
+ * Copyright (c) 2018-2023  Joachim Wiberg <troglobit@gmail.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,14 +31,18 @@
 
 #include "config.h"
 
+#include <ctype.h>
 #include <err.h>
 #include <errno.h>
 #include <getopt.h>
+#include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 
 #define SYSLOG_NAMES
 #include "compat.h"
@@ -60,8 +64,8 @@ static int create(char *path, mode_t mode, uid_t uid, gid_t gid)
  */
 static int logrotate(char *file, int num, off_t sz)
 {
-	int cnt;
 	struct stat st;
+	int cnt;
 
 	if (stat(file, &st))
 		return 1;
@@ -112,6 +116,48 @@ static int logrotate(char *file, int num, off_t sz)
 	return 0;
 }
 
+static void log_kmsg(FILE *fp, char *ident, int pri, int opts, char *buf)
+{
+	while (isspace(*buf))
+		buf++;
+
+	/* Always add [PID] so syslogd can find this later on */
+	fprintf(fp, "<%d>%s[%d]:%s\n", pri, ident, getpid(), buf);
+}
+
+static int nslookup(const char *host, const char *svcname, int family, struct sockaddr *sa)
+{
+	struct addrinfo hints, *ai, *result;
+	int error;
+
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_flags    = !host ? AI_PASSIVE : 0;
+	hints.ai_family   = family;
+	hints.ai_socktype = SOCK_DGRAM;
+
+	error = getaddrinfo(host, svcname, &hints, &result);
+	if (error == EAI_SERVICE) {
+		warnx("%s/udp: unknown service, trying syslog port 514", svcname);
+		svcname = "514";
+		error = getaddrinfo(host, svcname, &hints, &result);
+	}
+	if (error) {
+		warnx("%s (%s:%s)", gai_strerror(error), host, svcname);
+		return 1;
+	}
+
+	for (ai = result; ai; ai = ai->ai_next) {
+		if (ai->ai_family != AF_INET && ai->ai_family != AF_INET6)
+			continue;
+
+		memcpy(sa, ai->ai_addr, ai->ai_addrlen);
+		break;
+	}
+	freeaddrinfo(result);
+
+	return 0;
+}
+
 static int checksz(FILE *fp, off_t sz)
 {
 	struct stat st;
@@ -183,13 +229,23 @@ static int usage(int code)
 	       "\n"
 	       "Write MESSAGE (or line-by-line stdin) to syslog, or file (with logrotate).\n"
 	       "\n"
+	       "  -4        Prefer IPv4 address when sending remote, see -h\n"
+	       "  -6        Prefer IPv6 address when sending remote, see -h\n"
+	       "  -b        Use RFC3164 (BSD) style format, default: RFC5424\n"
 	       "  -c        Log to console (LOG_CONS) on failure\n"
 	       "  -d SD     Log SD as RFC5424 style 'structured data' in message\n"
 	       "  -f FILE   Log file to write messages to, instead of syslog daemon\n"
+	       "  -h HOST   Send (UDP) message to this remote syslog server (IP or DNS name)\n"
+	       "  -H NAME   Use NAME instead of system hostname in message header\n"
 	       "  -i        Log process ID of the logger process with each line (LOG_PID)\n"
+	       "  -I PID    Log process ID using PID, recommed using PID $$ for shell scripts\n"
+#ifdef __linux__
+	       "  -k        Log to kernel /dev/kmsg if /dev/log doesn't exist yet\n"
+#endif
 	       "  -m MSGID  Log message using this RFC5424 style MSGID\n"
 	       "  -n        Open log file immediately (LOG_NDELAY)\n"
 	       "  -p PRIO   Log message priority (numeric or facility.severity pair)\n"
+	       "  -P PORT   Use PORT (or named UDP service) for remote server, default: syslog\n"
 	       "  -r S[:R]  Enable log file rotation, default: 200 kB \e[4ms\e[0mize, 5 \e[4mr\e[0motations\n"
 	       "  -s        Log to stderr as well as the system log\n"
 	       "  -t TAG    Log using the specified tag (defaults to user name)\n"
@@ -207,20 +263,36 @@ static int usage(int code)
 
 int main(int argc, char *argv[])
 {
+	char *ident = NULL, *logfile = NULL;
+	char *host = NULL, *sockpath = NULL;
+	char *msgid = NULL, *sd = NULL;
+	char *svcname = "syslog";
+	off_t size = 200 * 1024;
+	int facility = LOG_USER;
+	int severity = LOG_NOTICE;
+	int family = AF_UNSPEC;
+	struct sockaddr sa;
+	int allow_kmsg = 0;
+	char buf[512] = "";
+	int log_opts = 0;
 	FILE *fp = NULL;
 	int c, num = 5;
-	int facility = LOG_USER;
-	int severity = LOG_INFO;
-	int log_opts = 0;
 	int rotate = 0;
-	off_t size = 200 * 1024;
-	char *ident = NULL, *logfile = NULL;
-	char *msgid = NULL, *sd = NULL;
-	char *sockpath = NULL;
-	char buf[512] = "";
 
-	while ((c = getopt(argc, argv, "?cd:f:im:np:r:st:u:v")) != EOF) {
+	while ((c = getopt(argc, argv, "46?bcd:f:h:H:iI:km:np:P:r:st:u:v")) != EOF) {
 		switch (c) {
+		case '4':
+			family = AF_INET;
+			break;
+
+		case '6':
+			family = AF_INET6;
+			break;
+
+		case 'b':
+			log_opts |= LOG_RFC3164;
+			break;
+
 		case 'c':
 			log_opts |= LOG_CONS;
 			break;
@@ -233,10 +305,31 @@ int main(int argc, char *argv[])
 			logfile = optarg;
 			break;
 
+		case 'h':
+			host = optarg;
+			break;
+
+		case 'H':
+			strlcpy(log.log_hostname, optarg, sizeof(log.log_hostname));
+			break;
+
 		case 'i':
 			log_opts |= LOG_PID;
 			break;
 
+		case 'I':
+			log_opts |= LOG_PID;
+			log.log_pid = atoi(optarg);
+			break;
+
+		case 'k':
+#ifdef __linux__
+			allow_kmsg = 1;
+#else
+			errx(1, "-k is not supported on non-Linux systems.");
+#endif
+			break;
+
 		case 'm':
 			msgid = optarg;
 			break;
@@ -250,6 +343,10 @@ int main(int argc, char *argv[])
 				return usage(1);
 			break;
 
+		case 'P':
+			svcname = optarg;
+			break;
+
 		case 'r':
 			parse_rotation(optarg, &size, &num);
 			if (size > 0 && num > 0)
@@ -269,7 +366,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'v':	/* version */
-			fprintf(stderr, "%s\n", version_info);
+			printf("%s\n", version_info);
 			return 0;
 
 		default:
@@ -286,7 +383,7 @@ int main(int argc, char *argv[])
 		while (optind < argc) {
 			size_t bytes;
 
-			bytes = snprintf(&buf[pos], len, "%s ", argv[optind++]);
+			bytes = snprintf(&buf[pos], len, "%s%s", pos ? " " : "", argv[optind++]);
 			pos += bytes;
 			len -= bytes;
 		}
@@ -308,6 +405,32 @@ int main(int argc, char *argv[])
 		if (access(sockpath, W_OK))
 			err(1, "Socket path %s", sockpath);
 		log.log_sockpath = sockpath;
+	} else if (allow_kmsg && access(_PATH_LOG, W_OK)) {
+		/*
+		 * -k and /dev/log is not yet up, user wants to prevent
+		 * logging to console and instead use the detour around
+		 * the kernel logger until syslogd has started.
+		 */
+		while (!access("/dev/kmsg", W_OK)) {
+			int pri = facility | severity;
+
+			fp = fopen("/dev/kmsg", "w");
+			if (!fp)
+				break;	/* fall back to log syslogp_r() */
+
+			if (!buf[0]) {
+				while ((fgets(buf, sizeof(buf), stdin)))
+					log_kmsg(fp, ident, pri, log_opts, chomp(buf));
+			} else
+				log_kmsg(fp, ident, pri, log_opts, buf);
+
+			return fclose(fp);
+		}
+	} else if (host) {
+		log.log_host = &sa;
+		if (nslookup(host, svcname, family, &sa))
+			return 1;
+		log_opts |= LOG_NDELAY;
 	}
 
 	openlog_r(ident, log_opts, facility, &log);
@@ -320,7 +443,7 @@ int main(int argc, char *argv[])
 
 	closelog_r(&log);
 
-	if (rotate && checksz(fp, size))
+	if (logfile && rotate && checksz(fp, size))
 		logrotate(logfile, num, size);
 
 	return 0;
